#
#Initialize once with the credentials
#Keep texting 'send_message' method to the number entered in the argument
#
#Dependencies -> 'twilio-ruby', 'metainspector'
#

require 'twilio-ruby' #The Twilio gem
require 'metainspector' #Gem to help extract the open graph data

module TextMe
	class Core
		#Constructor
		#What: Initialize Twilio with credentials and save the from number
		#Input: account_sid, auth_token => From twilio dashbaord
		# 		'from' number, 'to' number => from your twilio settings
		#Output: No output
		def initialize(account_sid, auth_token, sender_number, reciever_number)
			#Initialize the twilio client
			@client = Twilio::REST::Client.new account_sid, auth_token

			#Save the 'from' and 'to' number as private variable
			@sender_number = sender_number
			@reciever_number = reciever_number
		end

		#Helper Method
		#What: Take a youversion url and return the verse and the verse image from the open graph data
		#Input: You version url of the verse
		#Output: A hash with the (always) 'verse' => (verse name + verse text), 
		# 						 (optional) 'image_url' => image verse
		def meta_extractor(verse_id)
			jtj = TextMe::JTJVerseExtractor.new
			raw_data = jtj.extract_data(verse_id)

			bible_extractor = TextMe::BibleExtractor.new
			url = bible_extractor.verse_url_generator(raw_data["verse"])
			#Use a Hash to store the verse and image url
			metadata = Hash.new

			page = MetaInspector.new(url)
			
			#Get the verse name and the verse
			metadata['verse'] = page.meta['og:title']
			
			#If the page contains an opengraph image
			if not raw_data['image']
				if page.meta['og:image']
					if page.meta['og:image'].include? 'http'
						metadata['image_url'] = page.meta['og:image']
					end
				end
			else
				metadata['image_url'] = raw_data["image"]
			end

			return metadata
		end

		#What: Take the recievers number and the verse url. Calls the meta_extractor helper method
		#Input: 'To' number, You version url of the verse
		#Output: No actual output. Send's a text containing verse name(always), verse text(always) and verse image(optional) to the 'To' number
		def send_message(verse_id)
			#A call to the meta_extractor to get the verse name, verse text and verse image(optional)
			verse_data = meta_extractor(verse_id)

			#If: There is a verse image for the verse
			#Else: There is no image
			if verse_data['image_url']
				#Sending a text with the twilio client
				@client.api.account.messages.create(
					from: @sender_number,
					to: @reciever_number,
					body: verse_data['verse'],
					media_url: verse_data['image_url']
				)
			else
				#Sending a text with the twilio client
				@client.api.account.messages.create(
					from: @sender_number,
					to: @reciever_number,
					body: verse_data['verse']
				)

			end
		end
	end

	#Helper Class to generate the url from a verse name
	class BibleExtractor
		#Helper Method
		#What: Take in the verse name and return the metadata as a Hash.
		#Input: Verse name, for example 'Proverbs 3:5-6'. 
		# 		Can also accept chapter names eg. 'Romans 3' (not recommended due to the length of the chapters)
		#Output: Returns the book, chapter number and the specfic verse (optional) rfom the verse name
		def verse_meta_extractor(verse_name)
			book_list =["Genesis", "Exodus", "Leviticus", "Numbers", "Deuteronomy", "Joshua", "Judges", "Ruth", "1 Samuel", "2 Samuel", "1 Kings", "2 Kings", "1 Chronicles", "2 Chronicles", "Ezra", "Nehemiah", "Esther", "Job", "Psalms", "Proverbs", "Ecclesiastes", "Song of Songs", "Isaiah", "Jeremiah", "Lamentations", "Ezekiel", "Daniel", "Hosea", "Joel", "Amos", "Obadiah", "Jonah", "Micah", "Nahum", "Habakkuk", "Zephaniah", "Haggai", "Zechariah", "Malachi", "Matthew", "Mark", "Luke", "John", "Acts", "Romans", "1 Corinthians", "2 Corinthians", "Galatians", "Ephesians", "Philippians", "Colossians", "1 Thessalonians", "2 Thessalonians", "1 Timothy", "2 Timothy", "Titus", "Philemon", "Hebrews", "James", "1 Peter", "2 Peter", "1 John", "2 John", "3 John", "Jude", "Revelation"]
			verse_meta = {}

			#Loop through the book_list to see which book verse name contains
			for book in book_list
				if verse_name.downcase.include? book.downcase
					verse_meta["book"] = book

					#Split the rest of the verse name into list (chapter number + verse number)
					verse_number = verse_name.split(book)[1].strip

					#To check if the whole chapter is requested or specific verses
					if verse_number.include? ':'
						verse_meta['chapter_number'] = verse_number.split(':')[0].strip
						verse_meta['specific_verse'] = verse_number.split(':')[1].strip
					else
						verse_meta['chapter_number'] = verse_number
					end
				end
			end
			return verse_meta
		end

		#What: Is called from the Core class. Gets the verse name and calls verse_meta_extractor.
		# 	   Uses the meta data and book_name_formatter to generate th You Version url
		#Input: Verse name, for example 'Proverbs 3:5-6'. 
		# 		Can also accept chapter names eg. 'Romans 3' (not recommended due to the length of the chapters)
		#Output: You version url of the verse
		def verse_url_generator(verse_name)
			verse_meta = verse_meta_extractor(verse_name)
			url = 'https://bible.com/bible/59/' #base url
			url = url + book_name_formatter(verse_meta['book']) + '/' #book name
			url = url + verse_meta["chapter_number"] + '/' #chapter number
			#add verse number only if not just book name
			if verse_meta['specific_verse']
				url = url + verse_meta['specific_verse']
			end
			return url
		end

		#Helper Method
		#What: It is a very long switch statement (order maintained) that transform the bible book name
		#Input: Name of the specific book of the bible - eg. 'Psalms'
		#Output: The You Version url specific book name
		def book_name_formatter(book_name)
			transformed_name = nil

			case book_name
			when"Genesis"
				transformed_name = "gen"
			when"Exodus"
				transformed_name = "exo"		
			when"Leviticus"
				transformed_name = "lev"
			when"Numbers"
				transformed_name = "num"
			when"Deuteronomy"
				transformed_name = "deu"
			when"Joshua"
				transformed_name = "jos"
			when"Judges"
				transformed_name = "jdg"
			when"Ruth"
				transformed_name = "rut"
			when"1 Samuel"
				transformed_name = "1sa"
			when"2 Samuel"
				transformed_name = "2sa"
			when"1 Kings"
				transformed_name = "1ki"
			when"2 Kings"
				transformed_name = "2ki"
			when"1 Chronicles"
				transformed_name = "1ch"
			when"2 Chronicles"
				transformed_name = "2ch"
			when"Ezra"
				transformed_name = "ezr"
			when"Nehemiah"
				transformed_name = "neh"
			when"Esther"
				transformed_name = "est"
			when "Job"
				transformed_name = "job"
			when "Psalms"
				transformed_name = "psa"
			when "Proverbs"
				transformed_name = "pro"
			when "Ecclesiastes"
				transformed_name = "ecc"
			when "Song of Songs"
				transformed_name = "sng"
			when "Isaiah"
				transformed_name = "isa"
			when "Jeremiah"
				transformed_name = "jer"
			when "Lamentations"
				transformed_name = "lam"
			when "Ezekiel"
				transformed_name = "ezk"
			when "Daniel"
				transformed_name = "dan"
			when "Hosea"
				transformed_name = "hos"
			when "Joel"
				transformed_name = "jol"
			when "Amos"
				transformed_name = "amo"
			when "Obadiah"
				transformed_name = "oba"
			when "Jonah"
				transformed_name = "jon"
			when "Micah"
				transformed_name = "mic"
			when "Nahum"
				transformed_name = "nam"
			when "Habakkuk"
				transformed_name = "hab"
			when "Zephaniah"
				transformed_name = "zep"
			when "Haggai"
				transformed_name = "hag"
			when "Zechariah"
				transformed_name = "zec"
			when "Malachi"
				transformed_name = "mal"
			when "Matthew"
				transformed_name = "mat"
			when "Mark"
				transformed_name = "mek"
			when "Luke"
				transformed_name = "luk"
			when "John"
				transformed_name = "jhn"
			when "Acts"
				transformed_name = "act"
			when "Romans"
				transformed_name = "rom"
			when "1 Corinthians"
				transformed_name = "1co"
			when "2 Corinthians"
				transformed_name = "2co"
			when "Galatians"
				transformed_name = "gal"
			when "Ephesians"
				transformed_name = "eph"
			when "Philippians"
				transformed_name = "php"
			when "Colossians"
				transformed_name = "col"
			when "1 Thessalonians"
				transformed_name = "1th"
			when "2 Thessalonians"
				transformed_name = "2th"
			when "1 Timothy"
				transformed_name = "1ti"
			when "2 Timothy"
				transformed_name = "2ti"
			when "Titus"
				transformed_name = "tit"
			when "Philemon"
				transformed_name = "phm"
			when "Hebrews"
				transformed_name = "heb"
			when "James"
				transformed_name = "jas"
			when "1 Peter"
				transformed_name = "1pe"
			when "2 Peter"
				transformed_name = "2pe"
			when "1 John"
				transformed_name = "1jn"
			when "2 John"
				transformed_name = "2jn"
			when "3 John"
				transformed_name = "3jn"
			when "Jude"
				transformed_name = "jud"
			when "Revelation"
				transformed_name = "rev"
			end
		end
	end

	#Helper class to extract Join The Journey's (JTJ) daily devotion specific information
	class JTJVerseExtractor
		#Helper method
		#What: Makes a HTTP Get request to the JTJ API
		#Input: id of the JTJ entry
		#Output: the API response in JSON format
		def get_verse(entry_id)
			url = 'https://jtj.watermark.org/api/entries/' + entry_id.to_s
			result = Net::HTTP.get(URI.parse(url))
			return JSON.parse(result)
		end

		#What: Called from Core class to get the verse name and image link for specific JTJ
		#Input: id of the JTJ entry
		#Output: Hash containing the verse name and verse lock screen image 
		def extract_data(entry_id)
			#Getting the HTTP response in JSON format of the given JTJ entry
			data = get_verse(entry_id)

			result = Hash.new #Empty result hash
			
			#Get the lock screen image from the JSON
			if data["entry"]["links"]["lock_screen"]
				image_link = data["entry"]["links"]["lock_screen"]
				result["image"] = image_link
			end

			#Get the verse name from JSON
			if data["entry"]["memory_verse"]["reference"]
				verse_name = data["entry"]["memory_verse"]["reference"]
				result["verse"] = verse_name
			end

			return result
		end
	end
end